part of datagrid;

class _VirtualizingCellsWidget extends StatefulWidget {
  const _VirtualizingCellsWidget(
      {@required this.key, @required this.dataRow, this.isDirty})
      : super(key: key);
  final Key key;
  final DataRowBase dataRow;
  final bool isDirty;

  @override
  State<StatefulWidget> createState() => _VirtualizingCellsWidgetState();
}

class _VirtualizingCellsWidgetState extends State<_VirtualizingCellsWidget> {
  @override
  Widget build(BuildContext context) {
    final List<Widget> children =
        widget.dataRow._visibleColumns.map<Widget>((dataCell) {
      return dataCell._columnElement;
    }).toList(growable: false);

    return _VirtualizingCellsRenderObjectWidget(
      key: widget.key,
      dataRow: widget.dataRow,
      children: List.from(children),
      isDirty: widget.isDirty,
    );
  }
}

class _VirtualizingCellsRenderObjectWidget
    extends MultiChildRenderObjectWidget {
  _VirtualizingCellsRenderObjectWidget(
      {@required this.key, this.dataRow, this.isDirty, this.children})
      : super(key: key, children: RepaintBoundary.wrapAll(List.from(children)));

  final List<Widget> children;
  final DataRowBase dataRow;
  final Key key;
  final bool isDirty;

  @override
  _RenderVirtualizingCellsWidget createRenderObject(BuildContext context) {
    return _RenderVirtualizingCellsWidget(
        dataRow: this.dataRow, isDirty: this.isDirty);
  }

  @override
  void updateRenderObject(
      BuildContext context, _RenderVirtualizingCellsWidget renderObject) {
    super.updateRenderObject(context, renderObject);
    renderObject
      ..dataRow = this.dataRow
      ..isDirty = this.isDirty;
  }
}

class _VirtualizingCellWidgetParentData
    extends ContainerBoxParentData<RenderBox> {
  double width = 0.0;
  double height = 0.0;
  _VirtualizingCellWidgetParentData({this.width, this.height});

  void reset() {
    this.width = 0.0;
    this.height = 0.0;
    this.offset = Offset.zero;
  }
}

class _RenderVirtualizingCellsWidget extends RenderBox
    with
        ContainerRenderObjectMixin<RenderBox,
            _VirtualizingCellWidgetParentData>,
        RenderBoxContainerDefaultsMixin<RenderBox,
            _VirtualizingCellWidgetParentData> {
  bool get isDirty => this._isDirty;
  bool _isDirty = false;

  set isDirty(bool newValue) {
    this._isDirty = newValue;
    if (this._isDirty) {
      this.markNeedsLayout();
      this.markNeedsPaint();
    }

    this.dataRow?._isDirty = false;
  }

  DataRowBase get dataRow => this._dataRow;
  DataRowBase _dataRow;

  set dataRow(DataRowBase newValue) {
    if (this._dataRow == newValue) return;
    this._dataRow = newValue;
    this.markNeedsLayout();
    this.markNeedsPaint();
  }

  Rect get rowRect => this._measureRowRect();

  _RenderVirtualizingCellsWidget(
      {List<RenderBox> children, DataRowBase dataRow, bool isDirty})
      : this._dataRow = dataRow,
        this._isDirty = isDirty {
    addAll(children);
  }

  Rect _measureRowRect() {
    if (this.dataRow.isVisible) {
      final _DataGridSettings dataGridSettings =
          this.dataRow._dataGridStateDetails();
      final _VisualContainerHelper container = dataGridSettings.container;

      final _VisibleLineInfo lineInfo =
          container.scrollRows.getVisibleLineAtLineIndex(this.dataRow.rowIndex);

      final double lineSize = lineInfo != null ? lineInfo.size : 0.0;
      var origin = (lineInfo != null) ? lineInfo.origin : 0.0;

      origin += container.verticalOffset;

      if (this.dataRow.rowIndex >
          _GridIndexResolver.getHeaderIndex(dataGridSettings)) {
        origin -= dataGridSettings.headerRowHeight;
      }

      return Rect.fromLTWH(0, origin, container.extentWidth, lineSize);
    } else {
      return Rect.zero;
    }
  }

  @override
  bool get isRepaintBoundary => true;

  @override
  bool hitTestChildren(BoxHitTestResult result, {Offset position}) {
    return defaultHitTestChildren(result, position: position);
  }

  @override
  void setupParentData(RenderObject child) {
    super.setupParentData(child);
    if (child.parentData != null &&
        child.parentData != _VirtualizingCellWidgetParentData()) {
      child.parentData = _VirtualizingCellWidgetParentData();
    }
  }

  @override
  void performLayout() {
    void _layout({RenderBox child, double width, double height}) {
      child.layout(BoxConstraints.tightFor(width: width, height: height),
          parentUsesSize: true);
    }

    RenderBox child = this.firstChild;
    while (child != null) {
      final gridCell = child as _RenderGridCell;
      final _VirtualizingCellWidgetParentData _parentData = child.parentData;
      if (this.dataRow.isVisible && gridCell.dataCell.isVisible) {
        size = constraints.constrain(
            Size(this.constraints.maxWidth, this.constraints.maxHeight));
        final Rect columnRect =
            gridCell._measureColumnRect(this.constraints.maxHeight);
        _parentData.width = columnRect.width;
        _parentData.height = columnRect.height;
        _layout(
            child: child,
            width: _parentData.width,
            height: this.constraints.maxHeight);
        _parentData.offset = Offset(columnRect.left, columnRect.top);
      } else {
        size = constraints.constrain(Size.zero);
        child.layout(BoxConstraints.tightFor(width: 0.0, height: 0.0),
            parentUsesSize: false);
        _parentData.reset();
      }
      child = _parentData.nextSibling;
    }
  }

  @override
  void paint(PaintingContext context, Offset offset) {
    // Remove the below method if the mentioned report has resolved form framework side
    // https://github.com/flutter/flutter/issues/29702
    this._drawRowBackground(context, offset);

    RenderBox child = firstChild;
    while (child != null) {
      final _VirtualizingCellWidgetParentData childParentData =
          child.parentData;
      if (childParentData.width != 0.0 && childParentData.height != 0.0) {
        context.paintChild(child, childParentData.offset + offset);
      }
      child = childParentData.nextSibling;
    }

    this._drawCurrentRowBorder(context, offset);
  }

  Rect _getRowRect(_DataGridSettings dataGridSettings, Offset offset) {
    bool needToSetMaxConstraint() {
      return dataGridSettings.container.extentWidth <
              dataGridSettings.viewWidth &&
          dataGridSettings.textDirection == TextDirection.rtl;
    }

    Rect rect = Rect.fromLTWH(
        (needToSetMaxConstraint()
            ? this.constraints.maxWidth -
                min(dataGridSettings.container.extentWidth,
                    dataGridSettings.viewWidth) -
                (offset.dx + dataGridSettings.container.horizontalOffset)
            : offset.dx + dataGridSettings.container.horizontalOffset),
        offset.dy,
        needToSetMaxConstraint()
            ? this.constraints.maxWidth
            : min(dataGridSettings.container.extentWidth,
                dataGridSettings.viewWidth),
        constraints.maxHeight);

    return rect;
  }

  void _drawRowBackground(PaintingContext context, Offset offset) {
    final _DataGridSettings dataGridSettings =
        this.dataRow._dataGridStateDetails();
    var rect = _getRowRect(dataGridSettings, offset);
    var backgroundColor;
    if (dataGridSettings != null &&
        dataGridSettings.gridPaint != null &&
        this.dataRow != null)
      dataGridSettings.gridPaint..style = PaintingStyle.fill;

    if (this.dataRow.rowType == RowType.headerRow) {
      backgroundColor =
          dataGridSettings.dataGridThemeData.headerStyle.backgroundColor;
    } else {
      backgroundColor = this.dataRow.isSelectedRow
          ? dataRow.rowStyle != null &&
                  dataRow._stylePreference == StylePreference.styleAndSelection
              ? DataGridCellStyle.lerp(
                      dataGridSettings.dataGridThemeData.selectionStyle,
                      dataRow.rowStyle,
                      0.5)
                  .backgroundColor
              : dataGridSettings
                  .dataGridThemeData.selectionStyle.backgroundColor
          : dataRow.rowStyle != null
              ? dataRow.rowStyle.backgroundColor ??
                  dataGridSettings.dataGridThemeData.cellStyle.backgroundColor
              : dataGridSettings.dataGridThemeData.cellStyle.backgroundColor;
    }

    dataGridSettings.gridPaint?.color = backgroundColor;
    context.canvas.drawRect(rect, dataGridSettings.gridPaint);
  }

  void _drawCurrentRowBorder(PaintingContext context, Offset offset) {
    if (!kIsWeb) return;

    final _DataGridSettings dataGridSettings = dataRow._dataGridStateDetails();

    if (dataGridSettings.boxPainter != null &&
        dataGridSettings.selectionMode == SelectionMode.multiple &&
        dataGridSettings.navigationMode == GridNavigationMode.row &&
        dataGridSettings.currentCell.rowIndex == dataRow.rowIndex) {
      bool needToSetMaxConstraint() {
        return dataGridSettings.container.extentWidth <
                dataGridSettings.viewWidth &&
            dataGridSettings.textDirection == TextDirection.rtl;
      }

      final double stokeWidth = 1.0;
      final origin = (stokeWidth / 2 +
              dataGridSettings.dataGridThemeData.gridLineStrokeWidth)
          .ceil();

      final Rect rowRect = _getRowRect(dataGridSettings, offset);
      final maxWidth = needToSetMaxConstraint()
          ? rowRect.width - rowRect.left
          : rowRect.right - rowRect.left;

      var isHorizontalGridLinesEnabled =
          dataGridSettings.gridLinesVisibility == GridLinesVisibility.both ||
              dataGridSettings.gridLinesVisibility ==
                  GridLinesVisibility.horizontal;

      dataGridSettings.boxPainter.paint(
          context.canvas,
          Offset(rowRect.left + origin, rowRect.top + (origin / 2)),
          dataGridSettings.configuration.copyWith(
              size: Size(
                  maxWidth - (origin * 2),
                  this.constraints.maxHeight -
                      (origin * (isHorizontalGridLinesEnabled ? 1.5 : 1)))));
    }
  }
}

class _HeaderCellsWidget extends _VirtualizingCellsWidget {
  const _HeaderCellsWidget(
      {@required this.key, @required this.dataRow, this.isDirty})
      : super(key: key, dataRow: dataRow, isDirty: isDirty);

  final Key key;
  final DataRowBase dataRow;
  final bool isDirty;
}
