part of datagrid;

class _RowGenerator {
  List<DataRowBase> get items => this._items;
  List<DataRowBase> _items = [];
  set items(List<DataRowBase> newItems) => this._items = newItems;

  _DataGridStateDetails get dataGridStateDetails => this._dataGridStateDetails;
  _DataGridStateDetails _dataGridStateDetails;

  _VisualContainerHelper get container => this.dataGridStateDetails().container;

  _RowGenerator({_DataGridStateDetails dataGridStateDetails}) {
    this._dataGridStateDetails = dataGridStateDetails;
    this._items = [];
  }

  void _preGenerateRows(_VisibleLinesCollection visibleRows,
      _VisibleLinesCollection visibleColumns) {
    if (this.items.length != 0 ||
        this.dataGridStateDetails().container.rowCount <= 0) {
      return;
    }

    if (visibleRows != null) {
      for (int i = 0; i < visibleRows.length; i++) {
        var line = visibleRows[i];
        var dr;
        switch (line.region) {
          case _ScrollAxisRegion.header:
            dr = this._createHeaderRow(line.lineIndex, visibleColumns);
            break;
          case _ScrollAxisRegion.body:
            dr = this._createDataRow(line.lineIndex, visibleColumns);
            break;
          case _ScrollAxisRegion.footer:
            break;
        }

        if (dr != null) {
          this.items.add(dr);
        }

        dr = null;
        line = null;
      }
    }
  }

  void _ensureRows(_VisibleLinesCollection visibleRows,
      _VisibleLinesCollection visibleColumns) {
    var actualStartAndEndIndex = [];
    var region = RowRegion.header;

    List<DataRowBase> reUseRows() {
      return this
          .items
          .where((row) => ((row.rowIndex < 0 ||
                  row.rowIndex < actualStartAndEndIndex[0] ||
                  row.rowIndex > actualStartAndEndIndex[1]) &&
              !row._isEnsured))
          .toList(growable: false);
    }

    this.items.forEach((rows) {
      rows._isEnsured = false;
    });

    for (int i = 0; i < 3; i++) {
      if (i == 0) {
        region = RowRegion.header;
        actualStartAndEndIndex =
            this.container._getStartEndIndex(visibleRows, i);
      } else if (i == 1) {
        region = RowRegion.body;
        actualStartAndEndIndex =
            this.container._getStartEndIndex(visibleRows, i);
      } else {
        region = RowRegion.footer;
        actualStartAndEndIndex =
            this.container._getStartEndIndex(visibleRows, i);
      }

      for (int index = actualStartAndEndIndex[0];
          index <= actualStartAndEndIndex[1];
          index++) {
        var dr = this._indexer(index);
        if (dr == null) {
          var rows = reUseRows();
          if (rows != null) {
            this._updateRow(rows, index, region);
            rows = null;
          }
        }

        if (dr == null) {
          dr = this
              .items
              .firstWhere((row) => row.rowIndex == index, orElse: () => null);
        }

        if (dr != null) {
          if (!dr._isVisible) {
            dr._isVisible = true;
          }

          dr._isEnsured = true;
        } else {
          if (region == RowRegion.header) {
            dr = this._createHeaderRow(index, visibleColumns);
          } else {
            dr = this._createDataRow(index, visibleColumns);
          }

          if (dr != null) {
            dr._isEnsured = true;
            this.items.add(dr);
          }
        }

        dr = null;
      }
    }

    for (var row in this.items) {
      if (!row._isEnsured || row.rowIndex == -1) {
        row._isVisible = false;
      }
    }

    actualStartAndEndIndex = null;
    region = null;
  }

  void _ensureColumns(_VisibleLinesCollection visibleColumns) {
    this.items.forEach((row) {
      row._ensureColumns(visibleColumns);
    });
  }

  DataRowBase _createDataRow(
      int rowIndex, _VisibleLinesCollection visibleColumns,
      {RowRegion rowRegion = RowRegion.body}) {
    var dr = DataRow();
    dr._key = ObjectKey(dr);
    dr._dataGridStateDetails = this.dataGridStateDetails;
    dr.rowIndex = rowIndex;
    dr.rowRegion = rowRegion;
    dr.rowType = RowType.dataRow;
    this._checkQueryRowStyle(dr);
    this._checkForCurrentRow(dr);
    this._checkForSelection(dr);
    dr._initializeDataRow(visibleColumns);
    return dr;
  }

  DataRowBase _createHeaderRow(
      int rowIndex, _VisibleLinesCollection visibleColumns) {
    var dr = DataRow();
    dr._key = ObjectKey(dr);
    dr._dataGridStateDetails = this.dataGridStateDetails;
    dr.rowIndex = rowIndex;
    dr.rowRegion = RowRegion.header;
    dr.rowType = RowType.headerRow;
    dr._initializeDataRow(visibleColumns);
    return dr;
  }

  DataRowBase _indexer(int index) {
    for (int i = 0; i < this.items.length; i++) {
      if (this.items[i].rowIndex == index) {
        return this.items[i];
      }
    }

    return null;
  }

  void _updateRow(List<DataRowBase> rows, int index, RowRegion region) {
    if (region == RowRegion.header) {
      var dr = rows.firstWhere((row) => row.rowType == RowType.headerRow,
          orElse: () => null);
      if (dr != null) {
        dr._key = dr._key;
        dr.rowIndex = index;
        dr._dataGridStateDetails = this.dataGridStateDetails;
        dr.rowRegion = RowRegion.header;
        dr.rowType = RowType.headerRow;
        dr._rowIndexChanged();
        dr = null;
      } else {
        dr = _createHeaderRow(index,
            _SfDataGridHelper.getVisibleLines(this.dataGridStateDetails()));
        this.items.add(dr);
        dr = null;
      }
    } else {
      this._updateDataRow(rows, index, region);
    }
  }

  void _updateDataRow(List<DataRowBase> rows, int index, RowRegion region) {
    var row = rows?.firstWhere(
        (row) => row is DataRow && row.rowType == RowType.dataRow,
        orElse: () => null) as DataRow;

    if (row != null) {
      if (index < 0 || index >= this.container.scrollRows.lineCount) {
        row._isVisible = false;
      } else {
        row._key = row._key;
        row.rowIndex = index;
        row.rowRegion = region;
        this._checkQueryRowStyle(row);
        this._checkForCurrentRow(row);
        this._checkForSelection(row);
        row._rowIndexChanged();
      }
      row = null;
    } else {
      var dr = this._createDataRow(index,
          _SfDataGridHelper.getVisibleLines(this.dataGridStateDetails()));
      this.items.add(dr);
      dr = null;
    }
  }

  double _queryRowHeight(int rowIndex, double height) {
    var rowHeight = this.dataGridStateDetails().onQueryRowHeight(rowIndex);
    return rowHeight;
  }

  void _checkForSelection(DataRowBase row) {
    try {
      final _DataGridSettings dataGridSettings = this.dataGridStateDetails();
      final RowSelectionManager rowSelectionManager =
          dataGridSettings.rowSelectionManager;

      final int recordIndex = _GridIndexResolver.resolveToRecordIndex(
          dataGridSettings, row.rowIndex);
      final Object record =
          dataGridSettings.source.dataSource.elementAt(recordIndex);
      if (dataGridSettings.selectionMode != SelectionMode.none) {
        row._isSelectedRow = rowSelectionManager._selectedRows.contains(record);
      }
    } catch (e) {}
  }

  void _checkQueryRowStyle(DataRowBase dataRow) {
    final _DataGridSettings dataGridSettings = this._dataGridStateDetails();

    if (dataGridSettings.onQueryRowStyle != null) {
      QueryRowStyleArgs queryRowStyleArgs =
          QueryRowStyleArgs(rowIndex: dataRow.rowIndex);
      var rowStyle = dataGridSettings.onQueryRowStyle(queryRowStyleArgs);
      if (rowStyle != null) {
        dataRow.rowStyle = rowStyle;
        dataRow._stylePreference = queryRowStyleArgs.stylePreference;
      } else {
        dataRow.rowStyle = null;
        dataRow._stylePreference = StylePreference.selection;
      }
    } else {
      dataRow.rowStyle = null;
      dataRow._stylePreference = StylePreference.selection;
    }
  }

  void _checkForCurrentRow(DataRow dr) {
    final _DataGridSettings dataGridSettings = this.dataGridStateDetails();
    if (dataGridSettings.navigationMode == GridNavigationMode.cell) {
      final _CurrentCellManager currentCellManager =
          dataGridSettings.currentCell;

      getDataCell() {
        if (dr._visibleColumns.isEmpty) return null;
        var dc = dr._visibleColumns.firstWhere(
            (dataCell) =>
                dataCell.columnIndex == currentCellManager.columnIndex,
            orElse: () => null);
        return dc;
      }

      if (currentCellManager.rowIndex != -1 &&
          currentCellManager.columnIndex != -1 &&
          currentCellManager.rowIndex == dr.rowIndex) {
        var dataCell = getDataCell();
        currentCellManager._setCurrentCellDirty(dr, dataCell, true);
      } else if (dr.isCurrentRow) {
        var dataCell = getDataCell();
        currentCellManager._setCurrentCellDirty(dr, dataCell, false);
      } else {
        dr.isCurrentRow = false;
      }
    }
  }
}
