part of datagrid;

class _ScrollViewWidget extends StatefulWidget {
  const _ScrollViewWidget(
      {@required this.dataGridStateDetails, this.width, this.height});

  final _DataGridStateDetails dataGridStateDetails;
  final double width;
  final double height;

  @override
  State<StatefulWidget> createState() => _ScrollViewWidgetState();
}

class _ScrollViewWidgetState extends State<_ScrollViewWidget> {
  ScrollController _verticalController;
  ScrollController _horizontalController;
  FocusNode _dataGridFocusNode;
  double _width = 0.0;
  double _height = 0.0;
  double _scrollX = 0.0;
  double _scrollY = 0.0;
  bool _isScrolling = false;

  _DataGridSettings get _dataGridSettings => widget.dataGridStateDetails();

  _VisualContainerHelper get _container => _dataGridSettings.container;

  _RowGenerator get _rowGenerator => _dataGridSettings.rowGenerator;

  SelectionManagerBase get _rowSelectionManager =>
      _dataGridSettings.rowSelectionManager;

  void _verticalListener() {
    this.setState(() {
      final newValue = this._verticalController.offset;
      this._container.verticalOffset = newValue;
      this._scrollY = newValue;
      this._container.setRowHeights();
      this._isScrolling = true;
      this._container._isDirty = true;
    });
  }

  void _horizontalListener() {
    this.setState(() {
      final newValue = this._horizontalController.offset;
      this._container.horizontalOffset = newValue;
      this._scrollX = newValue;
      this._dataGridSettings.columnSizer._refresh(widget.width);
      this._isScrolling = true;
      this._container._isDirty = true;
    });
  }

  void _addScrollView(List<Widget> children) {
    final double _extentWidth = this._container.extentWidth;
    final double _headerRowHeight = this._dataGridSettings.headerRowHeight;
    final double _extentHeight =
        this._container.extentHeight - _headerRowHeight;

    final Widget scrollView = Scrollbar(
      controller: _horizontalController,
      child: SingleChildScrollView(
          controller: _horizontalController,
          scrollDirection: Axis.horizontal,
          physics: AlwaysScrollableScrollPhysics(),
          child: ConstrainedBox(
              constraints: BoxConstraints(
                minWidth: this._width,
              ),
              child: Scrollbar(
                controller: _verticalController,
                child: SingleChildScrollView(
                  scrollDirection: Axis.vertical,
                  controller: _verticalController,
                  physics: AlwaysScrollableScrollPhysics(),
                  child: ConstrainedBox(
                      constraints: BoxConstraints(
                        minHeight: min(this._height, _extentHeight),
                      ),
                      child: _VisualContainer(
                          isDirty: this._container._isDirty,
                          rowGenerator: this._rowGenerator,
                          containerSize: Size(
                              max(this._width, _extentWidth),
                              _extentHeight > this._height
                                  ? _extentHeight
                                  : min(this._height, _extentHeight)))),
                ),
              ))),
    );

    final Positioned wrapScrollView = Positioned.fill(
      child: scrollView,
      top: _headerRowHeight,
    );

    children.add(wrapScrollView);
  }

  void _addHeaderRows(List<Widget> children) {
    List<Widget> _buildHeaderRows() {
      return this
          ._rowGenerator
          .items
          .where((rows) => rows.rowRegion == RowRegion.header)
          .map<Widget>((dataRow) {
        return _HeaderCellsWidget(
          key: dataRow._key,
          dataRow: dataRow,
          isDirty: this._container._isDirty || dataRow._isDirty,
        );
      }).toList(growable: false);
    }

    if (this._rowGenerator.items.length > 0) {
      final List<Widget> headerRows = _buildHeaderRows();

      for (var row in headerRows) {
        Positioned header = Positioned.directional(
            textDirection: this._dataGridSettings.textDirection,
            start: (this._dataGridSettings.textDirection == TextDirection.ltr)
                ? -this._container.horizontalOffset
                : this._container.horizontalOffset,
            top: 0.0,
            height: this._dataGridSettings.headerRowHeight,
            width: this._width,
            child: row);
        children.add(header);
      }
    }
  }

  void _updateAxis() {
    final double width = this._width;
    final double height = this._height + this._dataGridSettings.headerRowHeight;
    this._container.updateAxis(Size(width, height));
  }

  void _ensureWidgets() {
    if (this._dataGridSettings.source.dataSource == null) return;

    if (!this._container._isPreGenerator) {
      this._container.preGenerateItems();
    } else {
      if (this._container._needToRefreshColumn) {
        this._ensureItems(true);
        this._container._needToRefreshColumn = false;
      } else {
        this._ensureItems(false);
      }
    }
  }

  void _ensureItems(bool needToRefresh) {
    final _VisibleLinesCollection visibleRows =
        this._container.scrollRows.getVisibleLines();
    final _VisibleLinesCollection visibleColumns =
        _SfDataGridHelper.getVisibleLines(
            this._rowGenerator.dataGridStateDetails());

    if (this._container._isGridLoaded && visibleColumns.length > 0) {
      this._rowGenerator._ensureRows(visibleRows, visibleColumns);
    }

    if (needToRefresh) {
      if (visibleColumns.length > 0) {
        this._rowGenerator._ensureColumns(visibleColumns);
      }
    }
  }

  void _handleSectionController() async {
    this.setState(() {});
  }

  bool handleFocusKeyOperation(FocusNode focusNode, RawKeyEvent e) {
    bool needToMoveFocus() {
      final _DataGridSettings dataGridSettings = this._dataGridSettings;

      bool canAllowToRemoveFocus(int rowIndex, int columnIndex) {
        return (dataGridSettings.navigationMode == GridNavigationMode.cell &&
                this._dataGridSettings.currentCell.rowIndex == rowIndex &&
                this._dataGridSettings.currentCell.columnIndex ==
                    columnIndex) ||
            (dataGridSettings.navigationMode == GridNavigationMode.row &&
                this._dataGridSettings.currentCell.rowIndex == rowIndex);
      }

      if (e.isShiftPressed) {
        var firstRowIndex =
            _SelectionHelper.getFirstRowIndex(_dataGridSettings);
        var firstCellIndex =
            _SelectionHelper.getFirstCellIndex(_dataGridSettings);

        if (canAllowToRemoveFocus(firstRowIndex, firstCellIndex)) {
          return false;
        } else {
          return true;
        }
      } else {
        var lastRowIndex =
            _SelectionHelper.getLastNavigatingRowIndex(_dataGridSettings);
        var lastCellIndex =
            _SelectionHelper.getLastCellIndex(_dataGridSettings);

        if (canAllowToRemoveFocus(lastRowIndex, lastCellIndex)) {
          return false;
        } else {
          return true;
        }
      }
    }

    if (e.logicalKey == LogicalKeyboardKey.tab) {
      return needToMoveFocus();
    } else {
      return true;
    }
  }

  void handleKeyOperation(RawKeyEvent e) {
    if (e.runtimeType == RawKeyDownEvent)
      this._rowSelectionManager.handleKeyEvent(e);
  }

  @override
  void initState() {
    this._verticalController = ScrollController(keepScrollOffset: true)
      ..addListener(_verticalListener);
    this._horizontalController = ScrollController(keepScrollOffset: true)
      ..addListener(_horizontalListener);

    this._height = widget.height;
    this._width = widget.width;
    this._dataGridSettings.verticalController = this._verticalController;
    this._dataGridSettings.horizontalController = this._horizontalController;

    if (this._dataGridSettings.rowSelectionManager != null) {
      this._dataGridSettings.rowSelectionManager
        ..addListener(_handleSectionController);
    }

    if (this._dataGridFocusNode == null) {
      this._dataGridFocusNode = FocusNode(onKey: handleFocusKeyOperation);
      this._dataGridSettings.dataGridFocusNode = this._dataGridFocusNode;
    }

    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    if (this._container._isDirty && !this._isScrolling) {
      this._updateAxis();
      this._updateColumnSizer();
      this._container.setRowHeights();
      this._container._needToRefreshColumn = true;
    }

    if (this._container._needToSetHorizontalOffset) this._setHorizontalOffset();

    if (this._container._isDirty) this._ensureWidgets();

    List<Positioned> children = [];

    this._addHeaderRows(children);

    this._addScrollView(children);

    this._container._isDirty = false;
    this._isScrolling = false;

    return RawKeyboardListener(
        focusNode: _dataGridFocusNode,
        onKey: handleKeyOperation,
        autofocus: false,
        child: Container(
            height: this._height,
            width: this._width,
            decoration: BoxDecoration(
              color: Colors.transparent,
            ),
            // Remove this ClipRect widget and uncomment the below line.
            // when the below mentioned issue resolved from framework side.
            // https://github.com/flutter/flutter/issues/50508
            // clipBehavior: Clip.antiAlias,
            child: ClipRect(
                clipBehavior: Clip.antiAlias,
                clipper: _DataGridClipper(),
                child: Stack(
                    fit: StackFit.passthrough,
                    overflow: Overflow.clip,
                    children: List.from(children)))));
  }

  @override
  void didUpdateWidget(_ScrollViewWidget oldWidget) {
    super.didUpdateWidget(oldWidget);

    if (oldWidget.width != widget.width ||
        oldWidget.height != widget.height ||
        this._container._needToSetHorizontalOffset) {
      this._width = widget.width;
      this._height = widget.height;
      this._container._needToSetHorizontalOffset = true;
      this._container._isDirty = true;
    }
  }

  @override
  void debugFillProperties(DiagnosticPropertiesBuilder properties) {
    super.debugFillProperties(properties);
    properties.add(DoubleProperty('_scrollY', _scrollY));
  }

  void _setHorizontalOffset() {
    if (this._container._needToSetHorizontalOffset) {
      this._container.horizontalOffset = this._scrollX;
      this._container.scrollColumns.markDirty();
    }

    this._container._needToSetHorizontalOffset = false;
  }

  void _updateColumnSizer() {
    final columnSizer = this._dataGridSettings.columnSizer;
    if (columnSizer != null) {
      if (columnSizer._isColumnSizerLoadedInitially) {
        columnSizer._initialRefresh(widget.width);
        columnSizer._isColumnSizerLoadedInitially = false;
      } else {
        columnSizer._refresh(widget.width);
      }
    }
  }

  @override
  void dispose() {
    if (this._verticalController != null) {
      this._verticalController.removeListener(this._verticalListener);
      this._verticalController.dispose();
    }

    if (this._horizontalController != null) {
      this._horizontalController.removeListener(this._horizontalListener);
      this._horizontalController.dispose();
    }

    super.dispose();
  }
}

// Remove the below custom clipper class.
// when the below mentioned issue resolved from framework side.
// https://github.com/flutter/flutter/issues/50508
class _DataGridClipper extends CustomClipper<Rect> {
  @override
  Rect getClip(Size size) {
    return Rect.fromLTWH(0.0, 0.0, size.width, size.height);
  }

  @override
  bool shouldReclip(_DataGridClipper oldClipper) {
    return false;
  }
}
