part of datagrid;

class _VisualContainerHelper {
  bool _isPreGenerator = false;
  bool _needToRefreshColumn = false;
  bool _isGridLoaded = false;
  int _headerLineCount = 1;
  bool _isDirty = false;

  // Used to set the horizontal offset for LTR to RTL and vise versa,
  bool _needToSetHorizontalOffset = false;

  _DataGridStateDetails get dataGridStateDetails =>
      this.rowGenerator.dataGridStateDetails;

  _RowGenerator get rowGenerator => this._rowGenerator;
  _RowGenerator _rowGenerator;

  set rowGenerator(_RowGenerator newValue) {
    _rowGenerator = newValue;
  }

  _RowHeightManager get rowHeightManager => this._rowHeightManager;
  _RowHeightManager _rowHeightManager = _RowHeightManager();

  set rowHeightManager(_RowHeightManager newValue) {
    _rowHeightManager = newValue;
  }

  _PaddedEditableLineSizeHostBase rowHeightsProvider;

  _PaddedEditableLineSizeHostBase columnWidthsProvider;

  _PaddedEditableLineSizeHostBase get rowHeights => this.rowHeightsProvider;

  _PaddedEditableLineSizeHostBase get columnWidths => this.columnWidthsProvider;

  _ScrollAxisBase get scrollRows {
    if (this._scrollRows == null) {
      this._scrollRows = this.createScrollAxis(
          true, this.verticalScrollBar, this.rowHeightsProvider);
      this.scrollRows.name = "ScrollRows";
    }

    return this._scrollRows;
  }

  _ScrollAxisBase _scrollRows;

  set scrollRows(_ScrollAxisBase newValue) => this._scrollRows = newValue;

  _ScrollAxisBase get scrollColumns {
    if (this._scrollColumns == null) {
      this._scrollColumns = this.createScrollAxis(
          true, this.horizontalScrollBar, this.columnWidthsProvider);
    }
    this._scrollColumns.name = "ScrollColumns";
    return this._scrollColumns;
  }

  _ScrollAxisBase _scrollColumns;

  set scrollColumns(_ScrollAxisBase newValue) => this._scrollColumns = newValue;

  _ScrollBarBase get horizontalScrollBar =>
      this._horizontalScrollBar ??
      (this._horizontalScrollBar = new _ScrollInfo());
  _ScrollBarBase _horizontalScrollBar;

  _ScrollBarBase get verticalScrollBar =>
      this._verticalScrollBar ?? (this._verticalScrollBar = new _ScrollInfo());
  _ScrollBarBase _verticalScrollBar;

  int get rowCount => this.rowHeightsProvider.lineCount;

  set rowCount(int newValue) {
    if (newValue > this.rowCount) {
      this.insertRows(this.rowCount, newValue - this.rowCount);
    } else if (newValue < this.rowCount) {
      this.removeRows(newValue, this.rowCount - newValue);
    }
  }

  int get columnCount => this.columnWidthsProvider.lineCount;

  set columnCount(int newValue) {
    if (newValue > this.columnCount) {
      this.insertColumns(this.columnCount, newValue - this.columnCount);
    } else if (newValue < this.columnCount) {
      this.removeColumns(newValue, this.columnCount - newValue);
    }
  }

  int get frozenRows => this.rowHeightsProvider.headerLineCount;

  set frozenRows(int newValue) {
    if (newValue < 0) {
      //throw new ArgumentOutOfRangeException("Negative values are not allowed");
    }

    this.rowHeightsProvider.headerLineCount = newValue;
  }

  double get extentWidth =>
      (this.scrollColumns as _PixelScrollAxis).totalExtent;

  double get extentHeight => (this.scrollRows as _PixelScrollAxis).totalExtent;

  _VisualContainerHelper({_RowGenerator rowGenerator})
      : _rowGenerator = rowGenerator {
    this.rowHeightsProvider = this.onCreateRowHeights();
    this.columnWidthsProvider = this.onCreateColumnWidths();
  }

  void setRowHeights() {
    if (this.rowGenerator == null) {
      return;
    }
    if (this.dataGridStateDetails().onQueryRowHeight == null) {
      return;
    }
    var visibleRows = this.scrollRows.getVisibleLines();

    int endIndex = 0;

    if (visibleRows.length < visibleRows.firstBodyVisibleIndex) {
      return;
    }

    endIndex = visibleRows[visibleRows.lastBodyVisibleIndex].lineIndex;

    int headerStart = 0;

    int headerEnd = this.scrollRows.headerLineCount - 1;
    this.rowHeightHelper(headerStart, headerEnd, RowRegion.header);

    rowHeightManager.updateRegion(headerStart, headerEnd, RowRegion.header);

    int footerStart =
        visibleRows.length > visibleRows.firstFooterVisibleIndex &&
                this.scrollRows.footerLineCount > 0
            ? visibleRows[visibleRows.firstFooterVisibleIndex].lineIndex
            : -1;
    int footerEnd = this.scrollRows.footerLineCount > 0
        ? this.scrollRows.lineCount - 1
        : -1;

    this.rowHeightHelper(footerStart, footerEnd, RowRegion.footer);

    rowHeightManager.updateRegion(footerStart, footerEnd, RowRegion.footer);

    var bodyStart = visibleRows[visibleRows.firstBodyVisibleIndex].origin;

    var bodyEnd = visibleRows[visibleRows.firstFooterVisibleIndex - 1].corner;

    var bodyStartLineIndex =
        visibleRows[visibleRows.firstBodyVisibleIndex].lineIndex;

    var current = bodyStart;
    var currentEnd = endIndex;

    (this.rowHeights as _LineSizeCollection).suspendUpdates();
    for (int index = bodyStartLineIndex;
        current <= bodyEnd && index < this.scrollRows.firstFooterLineIndex;
        index++) {
      var height = this.rowHeights[index][0];

      if (!rowHeightManager.contains(index, RowRegion.body)) {
        var rowHeight = this.rowGenerator._queryRowHeight(index, height);
        if (rowHeight != null) {
          height = rowHeight;
          this.rowHeights.setRange(index, index, height);
        }
      }

      current += height;
      currentEnd = index;
    }

    rowHeightManager.updateRegion(
        bodyStartLineIndex, currentEnd, RowRegion.body);

    if (rowHeightManager.dirtyRows.length > 0) {
      for (var index in rowHeightManager.dirtyRows) {
        if (index < 0 || index >= rowHeights.lineCount) continue;
        var height = this.rowHeights[index][0];
        var rowHeight = this.rowGenerator._queryRowHeight(index, height);
        if (rowHeight != null) {
          this.rowHeights.setRange(index, index, rowHeight);
        }
      }
      this.rowHeightManager.dirtyRows.clear();
    }
    (this.rowHeights as _LineSizeCollection).resumeUpdates();
    this.scrollRows.updateScrollBar(false);
  }

  void rowHeightHelper(int startIndex, int endIndex, RowRegion region) {
    if (startIndex < 0 || endIndex < 0) {
      return;
    }

    final _DataGridSettings dataGridSettings = this.dataGridStateDetails();
    for (int index = startIndex; index <= endIndex; index++) {
      if (!rowHeightManager.contains(index, region)) {
        var rowHeight = this.rowGenerator._queryRowHeight(index, 0);
        if (rowHeight != null) {
          this.rowHeights.setRange(index, index, rowHeight);

          if (region == RowRegion.header &&
              index == _GridIndexResolver.getHeaderIndex(dataGridSettings)) {
            dataGridSettings.headerRowHeight = rowHeight;
          }
        }
      }
    }
  }

  void preGenerateItems() {
    _VisibleLinesCollection visibleRows = this.scrollRows.getVisibleLines();
    _VisibleLinesCollection visibleColumns = _SfDataGridHelper.getVisibleLines(
        this.rowGenerator.dataGridStateDetails());

    if (visibleRows.length > 0 && visibleColumns.length > 0) {
      this.rowGenerator._preGenerateRows(visibleRows, visibleColumns);
      this._isPreGenerator = true;
    }
  }

  _PaddedEditableLineSizeHostBase onCreateRowHeights() {
    _LineSizeCollection lineSizeCollection = new _LineSizeCollection();
    return lineSizeCollection;
  }

  _PaddedEditableLineSizeHostBase onCreateColumnWidths() {
    _LineSizeCollection lineSizeCollection = new _LineSizeCollection();
    return lineSizeCollection;
  }

  void insertRows(int insertAtRowIndex, int count) {
    this.rowHeightsProvider.insertLines(insertAtRowIndex, count, null);
  }

  void removeRows(int removeAtRowIndex, int count) {
    this.rowHeightsProvider.removeLines(removeAtRowIndex, count, null);
  }

  void insertColumns(int insertAtColumnIndex, int count) {
    this.columnWidthsProvider.insertLines(insertAtColumnIndex, count, null);
  }

  void removeColumns(int removeAtColumnIndex, int count) {
    (this.columnWidths as _LineSizeCollection).suspendUpdates();
    this.columnWidthsProvider.removeLines(removeAtColumnIndex, count, null);
    (this.columnWidths as _LineSizeCollection).resumeUpdates();
  }

  void updateScrollBars() {
    this.scrollRows.updateScrollBar(false);
    this.scrollColumns.updateScrollBar(false);
  }

  void updateAxis(Size availableSize) {
    this.scrollRows.renderSize = availableSize.height;
    this.scrollColumns.renderSize = availableSize.width;
  }

  _ScrollAxisBase createScrollAxis(bool isPixelScroll, _ScrollBarBase scrollBar,
      _LineSizeHostBase lineSizes) {
    if (isPixelScroll) {
      return new _PixelScrollAxis.fromPixelScrollAxis(
          scrollBar, lineSizes, lineSizes as _DistancesHostBase);
    } else {
      return new _LineScrollAxis(scrollBar, lineSizes);
    }
  }

  _VisibleLineInfo getRowVisibleLineInfo(int index) {
    return this.scrollRows.getVisibleLineAtLineIndex(index);
  }

  double get horizontalOffset =>
      this.horizontalScrollBar.value - this.horizontalScrollBar.minimum;

  set horizontalOffset(double newValue) {
    if (this.dataGridStateDetails().textDirection == TextDirection.ltr) {
      this.horizontalScrollBar.value =
          newValue + this.horizontalScrollBar.minimum;
    } else {
      this.horizontalScrollBar.value = max(
              this.horizontalScrollBar.minimum,
              this.horizontalScrollBar.maximum -
                  this.horizontalScrollBar.largeChange) -
          newValue;
    }
    this._needToRefreshColumn = true;
  }

  double get verticalOffset =>
      this.verticalScrollBar.value - this.verticalScrollBar.minimum;

  set verticalOffset(double newValue) {
    if (this.verticalScrollBar.value !=
        (newValue + this.verticalScrollBar.minimum)) {
      this.verticalScrollBar.value =
          (newValue + this.verticalScrollBar.minimum);
    }
  }

  List<int> _getStartEndIndex(
      _VisibleLinesCollection visibleLines, int region) {
    var startIndex = 0;
    var endIndex = -1;
    switch (region) {
      case 0:
        if (visibleLines.firstBodyVisibleIndex > 0) {
          startIndex = 0;
          endIndex =
              visibleLines[visibleLines.firstBodyVisibleIndex - 1].lineIndex;
        }

        return [startIndex, endIndex];

        break;
      case 1:
        if ((visibleLines.firstBodyVisibleIndex <= 0 &&
                visibleLines.lastBodyVisibleIndex < 0) ||
            visibleLines.length <= visibleLines.firstBodyVisibleIndex) {
          return [startIndex, endIndex];
        } else {
          startIndex =
              visibleLines[visibleLines.firstBodyVisibleIndex].lineIndex;
          endIndex = visibleLines[visibleLines.lastBodyVisibleIndex].lineIndex;
          return [startIndex, endIndex];
        }
        break;
      case 2:
        if (visibleLines.firstFooterVisibleIndex < visibleLines.length) {
          startIndex =
              visibleLines[visibleLines.firstFooterVisibleIndex].lineIndex;
          endIndex = visibleLines[visibleLines.length - 1].lineIndex;
        }
        return [startIndex, endIndex];
        break;
      default:
        return [startIndex, endIndex];
        break;
    }
  }

  void _refreshDefaultLineSize() {
    this.rowHeights.defaultLineSize = this.dataGridStateDetails().rowHeight;
    this.columnWidths.defaultLineSize =
        this.dataGridStateDetails().defaultColumnWidth;
  }

  void _refreshHeaderLineCount() {
    this.dataGridStateDetails().headerLineCount = 1;
  }

  void _updateRowAndColumnCount() {
    if (this == null) {
      return;
    }

    var lineSizeCollection = (this.columnWidths as _LineSizeCollection);
    lineSizeCollection.suspendUpdates();
    this._updateColumnCount();
    this._updateRowCount();
    final _DataGridSettings dataGridSettings = this.dataGridStateDetails();
    if (this.rowCount > 0) {
      for (int i = 0;
          i <= _GridIndexResolver.getHeaderIndex(dataGridSettings);
          i++) {
        this.rowHeights[i] = dataGridSettings.headerRowHeight;
      }
    }

    //need to update the indent column width here
    lineSizeCollection.resumeUpdates();
    lineSizeCollection = null;
    this.updateScrollBars();
    //Need to reset the freeze span

    this.rowHeights.lineCount = this.rowCount;
    this.columnWidths.lineCount = this.columnCount;
  }

  void _updateColumnCount() {
    int columnCount = this.dataGridStateDetails().columns.length;
    this.columnCount = columnCount;
  }

  void _updateRowCount() {
    final _dataGrid = this.dataGridStateDetails();
    var lineSizeCollection = this.rowHeights as _LineSizeCollection;
    lineSizeCollection.suspendUpdates();
    int rowCount = 0;
    rowCount = _dataGrid.source.dataSource != null
        ? _dataGrid.source.dataSource.length
        : 0;
    rowCount += _dataGrid.headerLineCount;
    this.rowCount = rowCount;
    this._updateFreezePaneRows();
    //need to reset the hidden state
    lineSizeCollection.resumeUpdates();
    //need to check again need to call the updateFreezePaneRows here
    this._updateFreezePaneRows();
  }

  void _updateFreezePaneRows() {
    this.frozenRows = this._headerLineCount;
  }

  void _refreshView() {
    void resetRowIndex(DataRowBase dataRow) {
      dataRow.rowIndex = -1;
    }

    this.rowGenerator.items.forEach(resetRowIndex);
  }

  void _refreshViewStyle() {
    void updateColumn(DataCellBase dataCell) {
      dataCell._isDirty = true;
      dataCell._updateColumn();
    }

    this.rowGenerator.items.forEach((dataRow) {
      dataRow._isDirty = true;
      dataRow._visibleColumns.forEach(updateColumn);
    });
  }
}

class _RowHeightManager {
  _Range header = new _Range();
  _Range body = new _Range();
  _Range footer = new _Range();
  List<int> dirtyRows = new List<int>();

  bool contains(int index, RowRegion region) {
    _Range range;
    if (region == RowRegion.header) {
      range = this.header;
    } else if (region == RowRegion.body) {
      range = this.body;
    } else {
      range = this.footer;
    }

    if (range.isEmpty()) {
      if (this.dirtyRows.contains(index)) {
        this.dirtyRows.remove(index);
      }

      return false;
    }

    if (index >= range.start && index <= range.end) {
      if (this.dirtyRows.contains(index)) {
        this.dirtyRows.remove(index);
        return false;
      }

      return true;
    }

    return false;
  }

  void setDirty(int rowIndex) {
    if (!dirtyRows.contains(rowIndex)) dirtyRows.add(rowIndex);
  }

  _Range getRange(int index) {
    if (index == 0)
      return header;
    else if (index == 1)
      return body;
    else
      return footer;
  }

  void reset() {
    header.start =
        header.end = body.start = body.end = footer.start = footer.end = -1;
  }

  void resetBody() {
    body.start = body.end = -1;
  }

  void resetFooter() {
    footer.start = footer.end = -1;
  }

  void updateBody(int index, int count) {
    if ((index + count) <= body.start) {
      this.resetBody();
      return;
    } else if (index > body.end)
      return;
    else
      body.end = index;
  }

  void updateRegion(int start, int end, RowRegion region) {
    _Range range;
    if (region == RowRegion.header)
      range = header;
    else if (region == RowRegion.body)
      range = body;
    else
      range = footer;

    range.start = start;
    range.end = end;
  }
}

class _Range {
  int start = -1;
  int end = -1;
  _Range();
  bool isEmpty() {
    return start < 0 || end < 0;
  }
}
