part of datagrid;

class SelectionManagerBase extends ChangeNotifier {
  _SelectedRowCollection _selectedRows;

  _DataGridStateDetails _dataGridStateDetails;

  SelectionManagerBase({_DataGridStateDetails dataGridStateDetails}) {
    this._dataGridStateDetails = dataGridStateDetails;
    this._selectedRows = _SelectedRowCollection();
    this._selectedRows.selectedRow = [];
  }

  void handleTap(RowColumnIndex rowColumnIndex) {}

  void handleKeyEvent(RawKeyEvent keyEvent) {}

  void onGridSelectionModeChanged() {}

  void handleDataGridSourceChanges() {}

  void onSelectedRowChanged() {}

  void onSelectedIndexChanged() {}

  void onSelectedRowsChanged() {}

  void _onRowColumnChanged(int recordLength, int columnLength) {}

  void _updateSelectionController(
      {bool isSelectionModeChanged = false,
      bool isNavigationModeChanged = false,
      bool isDataSourceChanged = false}) {}

  bool _raiseSelectionChanging({List<Object> oldItems, List<Object> newItems}) {
    final _DataGridSettings dataGridSettings = this._dataGridStateDetails();
    if (dataGridSettings.onSelectionChanging == null) return true;

    return dataGridSettings.onSelectionChanging(newItems, oldItems) ?? true;
  }

  void _raiseSelectionChanged({List<Object> oldItems, List<Object> newItems}) {
    final _DataGridSettings dataGridSettings = this._dataGridStateDetails();
    if (dataGridSettings.onSelectionChanged == null) return;

    dataGridSettings.onSelectionChanged(newItems, oldItems);
  }

  int _getPreviousColumnIndex(
      _DataGridSettings dataGridSettings, int currentColumnIndex) {
    final _DataGridSettings dataGridSettings = this._dataGridStateDetails();
    int previousCellIndex = currentColumnIndex;
    GridColumn column = this
        ._getNextGridColumn(dataGridSettings, currentColumnIndex - 1, false);
    if (column != null)
      previousCellIndex = _GridIndexResolver.resolveToScrollColumnIndex(
          dataGridSettings, dataGridSettings.columns.indexOf(column));

    if (previousCellIndex == currentColumnIndex)
      previousCellIndex = currentColumnIndex - 1;

    return previousCellIndex;
  }

  int _getPreviousRowIndex(
      _DataGridSettings dataGridSettings, int currentRowIndex,
      {bool isSelection = false}) {
    var lastRowIndex =
        _SelectionHelper.getLastNavigatingRowIndex(dataGridSettings);
    if (currentRowIndex > lastRowIndex) return lastRowIndex;

    var firstRowIndex =
        _SelectionHelper.getFirstNavigatingRowIndex(dataGridSettings);
    if (currentRowIndex <= firstRowIndex) return firstRowIndex;

    var previousIndex = currentRowIndex;
    previousIndex = dataGridSettings.container.scrollRows
        .getPreviousScrollLineIndex(currentRowIndex);
    if (previousIndex == currentRowIndex) previousIndex = previousIndex - 1;

    return previousIndex;
  }

  GridColumn _getNextGridColumn(
      _DataGridSettings dataGridSettings, int columnIndex, bool moveToRight) {
    var resolvedIndex = _GridIndexResolver.resolveToGridVisibleColumnIndex(
        dataGridSettings, columnIndex);
    if (resolvedIndex < 0 || resolvedIndex >= dataGridSettings.columns.length)
      return null;

    var gridColumn = dataGridSettings.columns[resolvedIndex];
    if (gridColumn == null ||
        gridColumn.isHidden ||
        gridColumn.actualWidth == 0.0) {
      gridColumn = this._getNextGridColumn(dataGridSettings,
          moveToRight ? columnIndex + 1 : columnIndex - 1, moveToRight);
    }

    return gridColumn;
  }

  int _getNextColumnIndex(
      _DataGridSettings dataGridSettings, int currentColumnIndex) {
    int nextCellIndex = currentColumnIndex;
    int lastCellIndex = _SelectionHelper.getLastCellIndex(dataGridSettings);

    GridColumn column =
        this._getNextGridColumn(dataGridSettings, currentColumnIndex + 1, true);
    if (column != null) {
      var columnIndex = dataGridSettings.columns.indexOf(column);
      nextCellIndex = _GridIndexResolver.resolveToScrollColumnIndex(
          dataGridSettings, columnIndex);
    }

    if (nextCellIndex == currentColumnIndex)
      nextCellIndex = currentColumnIndex + 1;

    nextCellIndex =
        nextCellIndex > lastCellIndex ? lastCellIndex : nextCellIndex;
    return nextCellIndex;
  }

  int _getNextRowIndex(
      _DataGridSettings dataGridSettings, int currentRowIndex) {
    var lastRowIndex =
        _SelectionHelper.getLastNavigatingRowIndex(dataGridSettings);
    if (currentRowIndex >= lastRowIndex) return lastRowIndex;

    var firstRowIndex =
        _SelectionHelper.getFirstNavigatingRowIndex(dataGridSettings);

    if (currentRowIndex < firstRowIndex) return firstRowIndex;

    if (currentRowIndex >= dataGridSettings.container.scrollRows.lineCount)
      return -1;
    var nextIndex = 0;

    nextIndex = dataGridSettings.container.scrollRows
        .getNextScrollLineIndex(currentRowIndex);

    if (nextIndex == currentRowIndex) nextIndex = nextIndex + 1;

    return nextIndex;
  }

  @override
  bool operator ==(other) =>
      identical(this, other) ||
      other is SelectionManagerBase && runtimeType == other.runtimeType;

  @override
  int get hashCode => super.hashCode;
}
