part of datagrid;

/// Provides data for the [ILineSizeHost.LineSizeChanged] event.
class _RangeChangedArgs {
  int _from = 0;
  int _to = 0;
  double _oldSize = 0.0;
  double _newSize = 0.0;
  _RangeChangedArgs();

  /// Initializes a new instance of the RangeChangedArgs class.
  ///
  /// * from - _required_ - The start index.
  /// * to - _required_ - The end index.
  _RangeChangedArgs.fromArgs(int from, int to) {
    this._from = from;
    this._to = to;
  }

  /// Initializes a new instance of the RangeChangedArgs class.
  ///
  /// * from - _required_ - The start index.
  /// * to - _required_ - The end index.
  /// * oldSize - _required_ - The old size.
  /// * newSize - _required_ - The new size.
  _RangeChangedArgs.fromRangeChangedArgs(
      int from, int to, double oldSize, double newSize) {
    this._from = from;
    this._to = to;
    this._oldSize = oldSize;
    this._newSize = newSize;
  }

  /// Gets the start index.
  ///
  /// Returns the start index.
  int get from => this._from;

  /// Gets the end index.
  ///
  /// Returns the end index.
  int get to => this._to;

  /// Gets the old size.
  ///
  /// Returns the old size.
  double get oldSize => this._oldSize;

  /// Gets the new size.
  ///
  /// Returns the new size.
  double get newSize => this._newSize;
}

/// Provides data for the [ILineSizeHost.LinesRemoved] event.
class _LinesRemovedArgs {
  int _removeAt = 0;
  int _count = 0;
  _LinesRemovedArgs();

  /// Initializes a new instance of the [LinesRemovedArgs] class.
  ///
  /// * removeAt - _required_ - The index to remove.
  /// * count - _required_ - The count of the lines.
  _LinesRemovedArgs.fromArgs(int removeAt, int count) {
    this._removeAt = removeAt;
    this._count = count;
  }

  /// Gets the index to remove.
  ///
  /// Returns the index to remove.
  int get removeAt => this._removeAt;

  /// Gets the count of items to be removed.
  ///
  /// Returns the count of items to be removed.
  int get count => this._count;
}

/// Provides data for the [ILineSizeHost.LinesInserted] event.
class _LinesInsertedArgs {
  int _insertAt = 0;
  int _count = 0;
  _LinesInsertedArgs();

  /// Initializes a new instance of the [LinesInsertedArgs] class.
  ///
  /// * insertAt - _required_ - The index to insert.
  /// * count - _required_ - The count of the items to be inserted.
  _LinesInsertedArgs.fromArgs(int insertAt, int count) {
    this._insertAt = insertAt;
    this._count = count;
  }

  /// Gets the index to insert.
  ///
  /// Returns the index to insert.
  int get insertAt => this._insertAt;

  /// Gets the count of the items to be inserted.
  ///
  /// Returns the count of the items to be inserted.
  int get count => this._count;
}

/// Provides data for the [ILineSizeHost.DefaultLineSizeChanged] event.
class _DefaultLineSizeChangedArgs {
  double _oldValue = 0.0;
  double _newValue = 0.0;

  /// Initializes a new instance of the DefaultLineSizeChangedArgs class.
  _DefaultLineSizeChangedArgs();

  /// Initializes a new instance of the DefaultLineSizeChangedArgs class.
  ///
  /// * oldValue - _required_ - The old line size.
  /// * name - _required_ - newValue The new line size.
  _DefaultLineSizeChangedArgs.fromArgs(double oldValue, double newValue) {
    this._oldValue = oldValue;
    this._newValue = newValue;
  }

  /// Gets the old line size.
  ///
  /// Returns the old line size.
  double get oldValue => this._oldValue;

  /// Gets the new line size.
  ///
  /// Returns the new line size.
  double get newValue => this._newValue;
}

/// Provides data for the [ILineSizeHost.LineHiddenChanged] event.
class _HiddenRangeChangedArgs {
  int _from = 0;
  int _to = 0;
  bool _hide;

  _HiddenRangeChangedArgs();

  /// Initializes a new instance of the [HiddenRangeChangedArgs] class.
  ///
  /// * from - _required_ - The start index of the hidden range.
  /// * to - _required_ - The end index of the hidden range.
  /// * hide - _required_ - hide value
  _HiddenRangeChangedArgs.fromArgs(int from, int to, bool hide) {
    this._from = from;
    this._to = to;
    this._hide = hide;
  }

  /// Gets the start index of the hidden range.
  ///
  ///  Returns the start index of the hidden range.
  int get from => this._from;

  /// Gets the end index of the hidden range.
  ///
  /// Returns the end index of the hidden range.
  int get to => this._to;

  /// Gets a value indicating whether to hide the lines in the given range or not.
  ///
  /// Returns a boolean value indicating whether to hide the lines in the given range.
  bool get hide => this._hide;
}

/// Defines the various constants for the scroll changed action.
enum _ScrollChangedAction {
  /// - ScrollChangedAction.linesInserted, Specifies that the scroll is changed as the lines are inserted.
  linesInserted,

  /// - ScrollChangedAction.linesRemoved, Specifies that the scroll is changed as the lines are removed.
  linesRemoved,

  /// - ScrollChangedAction.footerLineCountChanged, Specifies that the scroll is changed as the footer line count is changed.
  footerLineCountChanged,

  /// - ScrollChangedAction.defaultLineSizeChanged, Specifies that the scroll is changed as the default line size is changed.
  defaultLineSizeChanged,

  /// - ScrollChangedAction.lineCountChanged, Specifies that the scroll is changed as the line count is changed.
  lineCountChanged,

  /// - ScrollChangedAction.hiddenLineChanged, Specifies that the scroll is changed as the header line is changed.
  hiddenLineChanged,

  /// - ScrollChangedAction.lineResized, Specifies that the scroll is changed as the lines are resized.
  lineResized,
}

/// Provides data for the `ScrollAxisBase.Changed` event.
class _ScrollChangedArgs {
  _ScrollChangedAction _scrollChangedAction;

  /// Gets the scroll changed action.
  ///
  /// Returns the `ScrollChangedAction`.
  _ScrollChangedAction get action => this._scrollChangedAction;

  _ScrollChangedArgs();

  /// Initializes a new instance of the ScrollChangedArgs class.
  ///
  /// * action - _required_ - The `ScrollChangedAction`.
  _ScrollChangedArgs.fromArgs(_ScrollChangedAction action) {
    this._scrollChangedAction = action;
  }
}
